<!DOCTYPE kpartgui>
<gui name="kategdb-plugin" library="kategdbplugin" version="1">
    <MenuBar>
        <Menu name="debug"><Text>&amp;Debug</Text>
            <Action name="targets" />
            <Action name="argLists" />
            <Separator/>
            <Action name="debug" />
            <Action name="kill" />
            <Action name="rerun" />
            <Separator/>
            <Action name="toggle_breakpoint" />
            <Action name="step_in" />
            <Action name="step_over" />
            <Action name="step_out" />
            <Action name="move_pc" />
            <Action name="run_to_cursor" />
            <Action name="continue" />
            <Separator/>
            <Action name="print_value" />
        </Menu>
    </MenuBar>

    <Menu name="ktexteditor_popup" noMerge="1">
        <Action name="popup_gdb"/>
    </Menu>

    <ToolBar name="gdbplugin">
        <text>GDB Plugin</text>
        <Action name="toggle_breakpoint"/>
        <Action name="step_in"/>
        <Action name="step_over"/>
        <Action name="step_out"/>
        <Action name="run_to_cursor"/>
        <Action name="continue"/>
    </ToolBar>

</gui>
