/**
 * This is a pseudo CSS file to test Kate's CSS syntax highlighting.
 */

@import url("othersheet.css") screen, tv;

body {
	font-size: 15pt;
	font-family: Verdana, Helvetica, "Bitstream Vera Sans", sans-serif;
	margin-top: 0px;			/* yet another comment */
	margin-bottom: 0px;
	// this is no comment, it's just broken!
	margin-left: 0px;
	margin-right: 0px;
}

.something
{
	margin-right: 0px;
	color: #cdd;
	color: #AAFE04;
	color: rgb(10%,30%,43%);
	background: maroon;
}

a:hover {
}

#header,
p.intro:first-letter,
p:lang(nl),
img[align="right"]
{
	border: 1px solid Qt::red !important;
	-moz-border-radius: 15px; /* unknown properties render italic */
}

@media print {

	#header
	{
		display: none;
	}

}

/*
TODO: add more tests, e.g. media
*/
