<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="ktexteditor_exporter" library="ktexteditor_exporter" version="1">

<MenuBar>
  <Menu name="file" noMerge="1"><text>&amp;File</text>
    <Action name="file_export_html" group="print_merge"/>
  </Menu>

  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="edit_copy_html" group="edit_paste_merge" />
  </Menu>
</MenuBar>

</gui>
