<!DOCTYPE kpartgui>
<gui name="katesearch" library="libkatesearchplugin" version="2">
    <MenuBar>
        <Menu name="edit">
            <text>&amp;Edit</text>
            <Action name="search_in_files" />
            <Action name="go_to_next_match" />
            <Action name="go_to_prev_match" />
        </Menu>
    </MenuBar>
</gui>
