<!DOCTYPE kpartgui>
<gui name="katehelloworld" library="libkatehelloworldplugin" version="2">
<MenuBar>
 <Menu name="tools"><text>&amp;Tools</text>
    <Action name="edit_insert_helloworld" />
 </Menu>
</MenuBar>
</gui>
