<!DOCTYPE kpartgui>
<gui name="katebuild-plugin" library="katebuildplugin" version="5">
    <MenuBar>
        <Menu name="Build"><text>&amp;Build</text>
            <Action name="run_make"/>
            <Action name="make_clean" />
            <Action name="quick_compile" />
            <Action name="stop" />
            <Separator/>
            <Action name="goto_prev" />
            <Action name="goto_next" />
            <Separator/>
            <Action name="targets" />
            <Action name="target_next" />
        </Menu>
    </MenuBar>
</gui>
