<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KatePartView" version="54">
<MenuBar>
  <Menu name="file" noMerge="1"><text>&amp;File</text>
    <Action name="file_save" group="save_merge" />
    <Action name="file_save_as" group="save_merge" />
    <Action name="file_reload" group="revert_merge" />
    <Action name="file_print" group="print_merge" />
    <Separator group="print_merge" />
  </Menu>

  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="edit_undo" group="edit_undo_merge" />
    <Action name="edit_redo" group="edit_undo_merge" />
    <Separator group="edit_undo_merge" />
    <Action name="edit_cut" group="edit_paste_merge" />
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="edit_paste" group="edit_paste_merge" />
    <Separator group="edit_paste_merge" />
    <Action name="edit_select_all" group="edit_select_merge" />
    <Action name="edit_deselect" group="edit_select_merge" />
    <Separator group="edit_select_merge" />
    <Action name="set_insert" group="edit_select_merge" />
    <Separator group="edit_select_merge" />
    <Action name="edit_find" group="edit_find_merge" />
    <Action name="edit_find_next" group="edit_find_merge" />
    <Action name="edit_find_prev" group="edit_find_merge" />
    <Action name="edit_replace" group="edit_find_merge" />
    <Action name="edit_find_selected" group="edit_find_merge" />
    <Action name="edit_find_selected_backwards" group="edit_find_merge" />
    <Separator group="edit_find_merge" />
    <Action name="go_goto_line" group="edit_find_merge"/>
  </Menu>

  <Menu name="view" noMerge="1"><text>&amp;View</text>
    <Action name="view_dynamic_word_wrap" group="view_operations" />
    <Action name="view_line_numbers" group="view_operations" />
    <Separator group="view_operations" />
    <Action name="view_inc_font_sizes" group="view_operations" />
    <Action name="view_dec_font_sizes" group="view_operations" />
  </Menu>

  <Menu name="tools" noMerge="1"><text>&amp;Tools</text>
    <Action name="tools_toggle_write_lock" group="tools_operations" />
    <Action name="set_encoding" group="tools_operations" />
    <Action name="add_bom" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="tools_toggle_automatic_spell_checking" group="tools_operations" />
    <Action name="tools_spelling" group="tools_operations" />
    <Action name="tools_spelling_from_cursor" group="tools_operations" />
    <Action name="tools_spelling_selection" group="tools_operations" />
    <Action name="tools_change_dictionary" group="tools_operations" />
    <Action name="tools_clear_dictionary_ranges" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="tools_cleanIndent" group="tools_operations" />
    <Action name="tools_align" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="tools_uppercase" group="tools_operations" />
    <Action name="tools_lowercase" group="tools_operations" />
    <Action name="tools_capitalize" group="tools_operations" />
    <Separator group="tools_operations" />
    <Action name="tools_join_lines" group="tools_operations" />
    <Action name="tools_apply_wordwrap" group="tools_operations" />
  </Menu>

  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="set_confdlg" group="configure_merge" />
  </Menu>
</MenuBar>

<Menu name="ktexteditor_popup" noMerge="0">
  <Action name="edit_cut" group="popup_operations" />
  <Action name="edit_copy" group="popup_operations" />
  <Action name="edit_paste" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="edit_select_all" group="popup_operations" />
  <Action name="edit_deselect" group="popup_operations" />
  <Separator group="popup_operations" />
  <Action name="spelling_suggestions" group="popup_operations" />
  <Separator group="popup_operations" />
</Menu>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_save" group="file_operations" />
  <Action name="file_save_as" group="file_operations" />
  <Action name="edit_undo" group="edit_operations" />
  <Action name="edit_redo" group="edit_operations" />
</ToolBar>

</kpartgui>
