<!DOCTYPE kpartgui>
<gui name="katectags-plugin" library="katectagsplugin" version="2">
    <MenuBar>
        <Menu name="CTags Menubar"><text>CTags</text>
            <Action name="ctags_return_step" />
            <Action name="ctags_lookup_current" />
            <Action name="ctags_lookup_current_as_declaration" />
            <Action name="ctags_lookup_current_as_definition" />
            <Action name="ctags_jump_to_next_matcht" />
        </Menu>
    </MenuBar>

    <Menu name="ktexteditor_popup" noMerge="1">
        <Action name="popup_ctags"/>
    </Menu>
</gui>

