<!DOCTYPE kpartgui>
<gui name="katexmltools" library="libkatexmltoolsplugin" version="8">
<MenuBar>
 <Menu name="xml"><text>&amp;XML</text>
    <Action name="xml_tool_insert_element" />
    <Action name="xml_tool_close_element" />
    <Action name="xml_tool_assign" />
 </Menu>
</MenuBar>
</gui>
