<!DOCTYPE kpartgui>
<gui name="ktexteditor_hlselection" library="ktexteditor_hlselection" version="1">
<MenuBar>
 <Menu name="tools"><text>&amp;Tools</text>
    <Separator group="tools_operations" />
    <Action name="tools_hl_selection" group="tools_operations"/>
 </Menu>
</MenuBar>
</gui>
