<!DOCTYPE kpartgui>
<gui name="kate_kttsd" library="kate_kttsd" version="3">
    <MenuBar>
        <Menu name="tools"><text>&amp;Tools</text>
            <Action name="tools_kttsd" />
        </Menu>
    </MenuBar>
    <ToolBar name="mainToolBar"><text>Main Toolbar</text>
        <Action name="tools_kttsd" />
    </ToolBar>
</gui>
