<!DOCTYPE kpartgui>
<gui name="katefiletreeplugin" library="katefiletreeplugin" version="6">

 <!-- merge in view -->
<MenuBar>
  <Menu name="view"><text>&amp;View</text>
    <Action name="filetree_prev_document" group="switch_document"/>
    <Action name="filetree_next_document" group="switch_document"/>
    <Action name="filetree_show_active_document" group="switch_document"/>
  </Menu>
</MenuBar>

<!-- merge in main toolbar -->
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
    <Action name="filetree_prev_document" group="switch_document"/>
    <Action name="filetree_next_document" group="switch_document"/>
</ToolBar>

</gui>

<!-- kate: space-indent on; indent-width 2; replace-tabs on; -->
