<!DOCTYPE kpartgui>
<gui name="kateexternaltools" library="kateexternaltoolsplugin" version="2">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_mail" group="export_merge"/>
 </Menu>
 <Menu name="tools"><text>&amp;Tools</text>
    <Action name="tools_external" />
 </Menu>
</MenuBar>
</gui>
