#ifndef KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT_H
#define KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT_H

/* needed for KDE_EXPORT macros */
#include <kdemacros.h>

#ifndef SNIPPET_EDITOR
# ifdef MAKE_KTEXTEDITOR_CODESNIPPETS_CORE_LIB
#  define KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT KDE_EXPORT
# else
#  define KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT KDE_IMPORT
# endif
#else
# define KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT
#endif

#endif /*KTEXTEDITOR_CODESNIPPETS_CORE_EXPORT_H*/
