<!DOCTYPE kpartgui>
<kpartplugin name="ktexteditor_insanehtml_le" library="ktexteditor_insanehtml_le" version="1">
<MenuBar>
 <Menu name="tools"><Text>&amp;Tools</Text>
    <separator group="tools_operations" />
    <Action name="tools_insanehtml_le" group="tools_operations"/>
 </Menu>
</MenuBar>
</kpartplugin>
